# Dummy module that allows to say "install Symphero::Data' in CPAN
# shell. Let me know if there is better way to do that <am@xao.com>
####################################################################
package Symphero::Data;
1;
__END__

=head1 NAME

Symphero::Data - low-level data manipulation modules for Symphero E-Commerce suite.

=head1 SYNOPSIS

None, see individual manpages.

=head1 DESCRIPTION

Symphero::Data is actually a dummy package that includes the following
actual modules:

=over

=item * Symphero::MultiValueDB

=item * Symphero::MultiValueHash

=item * Symphero::SimpleHash

=item * Symphero::Utils

=back

=head1 EXPORTS

Nothing.

=head1 AUTHOR

Brave New Worlds, Andrew Maltsev <amaltsev@xao.com>

=head1 SEE ALSO

Suggested reading:
L<Symphero::MultiValueDB>,
L<Symphero::MultiValueHash>,
L<Symphero::SimpleHash>,
L<Symphero::Utils>.

=cut
